@@ -597,6 +597,68 @@ pllua_runstring(lua_State *L, const char *chunkname, const char *str, bool use_s
 	}
 }
 
+/*
+ * Wrapper for stack depth checks of dangerous Lua functions.
+ *
+ * Original function is in upvalue 1. Unfortunately we can't tail call, but in
+ * fact this has the slight benefit that we add another layer of increments to
+ * nCcalls, halving the maximum possible recursion depth for the wrapped
+ * funcs.
+ */
+
+static int
+pllua_stack_check_wrapper(lua_State *L)
+{
+	int nargs = lua_gettop(L);
+	PLLUA_CHECK_PG_STACK_DEPTH();
+	lua_pushvalue(L, lua_upvalueindex(1));
+	lua_insert(L, 1);
+	lua_call(L, nargs, LUA_MULTRET);
+	return lua_gettop(L);
+}
+
+/*
+ * List of functions we have to add extra stack checks to.
+ */
+struct wrapper_info {
+	const char *name;
+	const char *libname;
+};
+static struct wrapper_info stack_wrap_list[] = {
+	/* no functions in base lib; pcall/xpcall handled in error.c */
+	{ NULL, "string" },
+	{ "format", NULL },		/* invokes metamethods */
+	{ "gsub", NULL },		/* invokes a passed-in function value */
+	{ NULL, "table" },
+	{ "concat", NULL },		/* invokes metamethods */
+	{ NULL, NULL }
+};
+
+static void pllua_wrap_stack_checks(lua_State *L)
+{
+	struct wrapper_info *p;
+
+	lua_getfield(L, LUA_REGISTRYINDEX, "_LOADED");
+	lua_pushglobaltable(L);
+
+	for (p = stack_wrap_list; p->name || p->libname; ++p)
+	{
+		if (p->libname)
+		{
+			lua_getfield(L, -2, p->libname);
+			lua_replace(L, -2);
+		}
+		if (p->name)
+		{
+			lua_getfield(L, -1, p->name);
+			lua_pushcclosure(L, pllua_stack_check_wrapper, 1);
+			lua_setfield(L, -2, p->name);
+		}
+	}
+
+	lua_pop(L, 2);
+}
+
 /*
  * Lua-environment part of interpreter setup.
  *
@@ -645,10 +707,17 @@ pllua_init_state_phase1(lua_State *L)
 
 	luaL_openlibs(L);
 
+	/*
+	 * Apply stack-checking wrappers to standard library functions that have
+	 * excessive stack use. Must be done after openlibs but before trusted
+	 * setup.
+	 */
+	pllua_wrap_stack_checks(L);
+
 	/*
 	 * Initialize our error handling, which replaces many base functions
-	 * (pcall, xpcall, error, assert). Must be done after openlibs but before
-	 * anything might throw a pg error.
+	 * (pcall, xpcall, etc.). Must be done after openlibs but before anything
+	 * might throw a pg error.
 	 */
 	luaL_requiref(L, "pllua.error", pllua_open_error, 0);
 
@@ -659,6 +728,12 @@ pllua_init_state_phase1(lua_State *L)
 	 */
 	luaL_requiref(L, "pllua.print", pllua_open_print, 0);
 
+	/*
+	 * Paths module does no db access and is safe in postmaster, and on_init
+	 * strings have a legitimate need for it.
+	 */
+	luaL_requiref(L, "pllua.paths", pllua_open_paths, 0);
+
 	/*
 	 * Early init of the trusted sandbox, so that on_init can do trusted setup
 	 * (even though we don't know in on_init whether we're trusted or not).
@@ -724,6 +799,8 @@ pllua_init_state_phase2(lua_State *L)
 
 	luaL_requiref(L, "pllua.jsonb", pllua_open_jsonb, 0);
 
+	luaL_requiref(L, "pllua.time", pllua_open_time, 0);
+
 	/*
 	 * complete the initialization of the trusted-mode sandbox.
 	 * We do this in untrusted interps too, but for those, we don't
