@@ -22,8 +22,8 @@
 
 #define PLLUA_VERSION_STR "2.0"
 #define PLLUA_VERSION_NUM 200
-#define PLLUA_REVISION_STR "2.0001"
-#define PLLUA_REVISION_NUM 20001
+#define PLLUA_REVISION_STR "2.0004"
+#define PLLUA_REVISION_NUM 20004
 
 /* PG version dependencies */
 #include "pllua_pgver.h"
@@ -33,13 +33,13 @@
 /* assume Lua 5.1 is actually luajit */
 #include "pllua_luajit.h"
 
-#elif LUA_VERSION_NUM >= 503
+#elif LUA_VERSION_NUM == 503
 
 /* Lua version dependencies */
 #include "pllua_luaver.h"
 
 #else
-#error Unsupported Lua version (only Lua 5.3+ and Luajit are supported)
+#error Unsupported Lua version (only Lua 5.3 and Luajit are supported)
 #endif
 
 /*
@@ -125,7 +125,6 @@ pllua_setcontext(pllua_context_type newctx)
 #define PLLUA_CHECK_PG_STACK_DEPTH()							\
 	do { if (stack_is_too_deep()) luaL_error(L, "stack depth exceeded"); } while (0)
 
-
 /*
  * Describes one call to the top-level handler.
  */
@@ -476,6 +475,8 @@ pllua_interpreter *pllua_getstate(bool trusted, pllua_activation_record *act);
 pllua_interpreter *pllua_getinterpreter(lua_State *L);
 int pllua_set_new_ident(lua_State *L);
 void pllua_run_extra_gc(lua_State *L, unsigned long gc_debt);
+PGDLLEXPORT bool pllua_stack_is_too_deep(void);
+PGDLLEXPORT void pllua_stack_depth_error(void);
 
 extern bool pllua_track_gc_debt;
 
@@ -644,6 +645,9 @@ void pllua_pgfunc_new(lua_State *L);
 FmgrInfo *pllua_pgfunc_init(lua_State *L, int nd, Oid fnoid, int nargs, Oid *argtypes, Oid rettype);
 void pllua_pgfunc_table_new(lua_State *L);
 
+/* paths.c */
+int pllua_open_paths(lua_State *L);
+
 /* preload.c */
 int pllua_preload_compat(lua_State *L);
 
@@ -657,6 +661,9 @@ int pllua_cursor_cleanup_portal(lua_State *L);
 int pllua_spi_newcursor(lua_State *L);
 int pllua_cursor_name(lua_State *L);
 
+/* time.c */
+int pllua_open_time(lua_State *L);
+
 /* trigger.c */
 int pllua_open_trigger(lua_State *L);
 
